package fullstack

import (
	"testing"
	"time"

	raven "github.com/getsentry/raven-go"
	"github.com/stretchr/testify/assert"
	"xbus.io/go-xbus/api"
)

type TestTransport struct {
	t       *testing.T
	packets chan *raven.Packet
}

func NewTestTransport(t *testing.T) *TestTransport {
	return &TestTransport{
		t:       t,
		packets: make(chan *raven.Packet, 5),
	}
}

// Send stores the packet for later inspection
func (tt *TestTransport) Send(url, authHeader string, packet *raven.Packet) error {
	tt.packets <- packet
	return nil
}

func (tt *TestTransport) Close() {
	close(tt.packets)
}

func TestRaven(t *testing.T) {
	t.Run("CaptureProcessERROR", func(t *testing.T) {
		ft := InitFullstack(t, false)
		if ft == nil {
			return
		}

		transport := NewTestTransport(t)
		defer transport.Close()

		ft.Server.SetRavenTransport(transport)
		if !ft.ServerStart() {
			t.Fatal("Server did not start")
		}
		defer ft.Shutdown()

		ft.CreateRole(api.Actor_EMITTER, "anyemitter")
		_, emitter := ft.CreateEmitter("bob", []string{"anyemitter"})

		_, err := emitter.SendItems(nil, api.DefaultOutput, "evt1", []api.Item{[]byte("item1")})

		if err == nil {
			t.Fatal("Should get a 'no matching pipeline' error")
		}
		assert.Equal(t, "No matching pipeline", err.Error())

		select {
		case packet := <-transport.packets:
			assert.Regexp(t, "\\[process\\]\\[from bob\\] No matching pipeline", packet.Message)
		case <-time.After(2 * time.Second):
			t.Error("Timeout: no raven packet sent")
		}
	})
}
